/** @file readTsa.C
 ** @author Florian Uhlig <f.uhlig@gsi.de>
 ** @since August 2014
 ** @date 08.08.2014
 **
 ** ROOT macro to read tsa files which have been generated by a flib board and convert the data
 ** into cbmroot format.
 ** Uses CbmFlibFileSource as source task.
 */


void readTsaTimeOnline(Bool_t highP = true)  //TString inFile =
                                             //"data/98_ba2015.tsa"
                                             //"data/129_ba2015.tsa"
                                             //	      "data/22_sps2016.tsa"
                                             //"data/1076_cern2014.tsa"
                                             //"data/test.tsa"
                                             //	      )
{

  // --- Specify input file name (this is just an example)
  //TString inFile = "spadic_dlm_trigger_2014-11-15_noepoch.tsa";
  //TString inFile1 = "spadic_noise_trigger_2014-11-15_withepoch.tsa";

  // --- Specify number of events to be produced.
  // --- -1 means run until the end of the input file.
  Int_t nEvents = -1;

  // --- Specify output file name (this is just an example)
  TString outFile = "data/online.root";
  // --- Set log output levels
  FairLogger::GetLogger()->SetLogScreenLevel("INFO");
  FairLogger::GetLogger()->SetLogVerbosityLevel("LOW");

  // --- Set debug level
  gDebug = 0;

  std::cout << std::endl;
  //std::cout << ">>> readTsa:  input file is " << inFile  << std::endl;
  std::cout << ">>> readTsa: output file is " << outFile << std::endl;

  // ========================================================================
  // ========================================================================

  std::cout << std::endl;
  std::cout << ">>> readTsa: Initialising..." << std::endl;

  // Spadic Unpacker
  CbmTSUnpackSpadic11OnlineMonitor* spadic_unpacker = new CbmTSUnpackSpadic11OnlineMonitor(highP);

  // --- Source task
  CbmFlibFileSourceNew* source = new CbmFlibFileSourceNew();

  //source->AddFile(inFile1);
  source->AddUnpacker(spadic_unpacker, 0x40);
  source->SetHostName("flip00");

  // --- Run
  FairRunOnline* run = new FairRunOnline(source);
  run->SetSource(source);
  run->SetOutputFile(outFile);
  run->ActivateHttpServer(100);
  run->SetAutoFinish(kFALSE);
  //  run->SetEventHeader(event);

  //  gDebug=2;
  //FairTask* spadicRawBeam = new CbmTrdTimeCorrel();
  //run->AddTask(spadicRawBeam);

  run->Init();


  // --- Start run
  TStopwatch timer;
  timer.Start();
  std::cout << ">>> readTsa: Starting run..." << std::endl;
  run->Run(nEvents, 0);  // run until end of input file
  timer.Stop();

  // --- End-of-run info
  Double_t rtime = timer.RealTime();
  Double_t ctime = timer.CpuTime();
  std::cout << std::endl << std::endl;
  std::cout << ">>> readTsa: Macro finished successfully." << std::endl;
  std::cout << ">>> readTsa: Output file is " << outFile << std::endl;
  std::cout << ">>> readTsa: Real time " << rtime << " s, CPU time " << ctime << " s" << std::endl;
  std::cout << std::endl;

  /// --- Screen output for automatic tests
  std::cout << " Test passed" << std::endl;
  std::cout << " All ok " << std::endl;
}
