/** @file fit_speed.C
  * @copyright Copyright (C) 2009-2010 GSI/JINR-LIT
  * @license SPDX-License-Identifier: GPL-3.0-only
  * @authors Andrey Lebedev [orginator] **/

void fit_speed(Int_t nEvents = 10000)
{
  TString dir              = "/d/cbm02/andrey/std13_10mu/";
  TString mcFile           = dir + "mc.0000.root";
  TString globalTracksFile = dir + "global.tracks.ideal.0000.root";
  TString parFile          = dir + "param.0000.root";
  TString outFile          = dir + "fit.speed.0000.root";

  TStopwatch timer;
  timer.Start();

  gROOT->LoadMacro("$VMCWORKDIR/gconfig/basiclibs.C");
  basiclibs();
  gROOT->LoadMacro("$VMCWORKDIR/macro/littrack/cbmrootlibs.C");
  cbmrootlibs();

  FairRunAna* run = new FairRunAna();
  run->SetInputFile(mcFile);
  run->AddFriend(globalTracksFile);
  run->SetOutputFile(outFile);

  // -------------------------------------------------------------------------
  CbmLitTrackFitSpeedTest* test = new CbmLitTrackFitSpeedTest();
  test->SetNofPlanes(13);
  test->SetNofTrdHits(0);
  test->SetNofMuchHits(13);
  run->AddTask(test);
  // -------------------------------------------------------------------------
  TString parDir      = TString(gSystem->Getenv("VMCWORKDIR")) + TString("/parameters");
  TString stsDigiFile = parDir + "/sts/sts_standard.digi.par";
  // -----  Parameter database   --------------------------------------------
  FairRuntimeDb* rtdb        = run->GetRuntimeDb();
  FairParRootFileIo* parIo1  = new FairParRootFileIo();
  FairParAsciiFileIo* parIo2 = new FairParAsciiFileIo();
  parIo1->open(parFile.Data());
  parIo2->open(stsDigiFile.Data(), "in");
  rtdb->setFirstInput(parIo1);
  rtdb->setSecondInput(parIo2);
  rtdb->setOutput(parIo1);
  rtdb->saveOutput();
  // ------------------------------------------------------------------------

  // -----   Intialise and run   --------------------------------------------
  run->Init();
  run->Run(0, nEvents);
  // ------------------------------------------------------------------------

  // -----   Finish   -------------------------------------------------------
  timer.Stop();
  cout << endl << endl;
  cout << "Macro finished succesfully." << endl;
  cout << "Output file is " << outFile << endl;
  cout << "Parameter file is " << parFile << endl;
  cout << "Real time " << timer.RealTime() << " s, CPU time " << timer.CpuTime() << " s" << endl;
  cout << endl;
  // ------------------------------------------------------------------------
}
